% FIGURE 4
% compute generalized eigen-values for excess power mon and alf

clear

% -- set plot context
debug=0;

graw('reset;set data style lines;set pointsize .5;') 
gset term x11
gset nokey
gset lmargin 9;
gset rmargin 1;

fs=20e3; % Hz
ncol=512;

% -- eps, parameters
jmin=13;
jmax=26;

% -- alf, parameters
Nalf=40;

% -- noise spectra and corr function

nf=2^(nextpow2(ncol));
psd=zeros(nf,1);
acftmp=zeros(nf,1);
f=(0:nf/2)/nf*fs;

% use of official Virgo sensitivity
% .5 factor because the given curve is one sided
psd(1:nf/2+1)=.5*(9e-37./f.^5+4.5e-43./f+3.24e-46*(1+(f/500).^2));
psd(1)=psd(2);
psd(nf/2+2:nf)=flipud(psd(2:nf/2));
acftmp=fs*real(ifft(psd));

hnoisef=sqrt(psd);
wfilter=fs*real(ifft(hnoisef));

acf=acftmp(1:ncol);

clear psd acftmp

% -- eps

fprintf('computing noise corr matrix...');
fflush(1);
R=toeplitz(acf);
fprintf(' done.\n');

% -- cholesky
fprintf('computing cholesky...');
fflush(1);
g=chol(R);
fprintf(' done.\n');

fprintf('computing inverse...');
fflush(1);
g=inv(g);
fprintf(' done.\n');

% -- compute covariance matrix
fprintf('computing kernel matrix...');
fflush(1);
Heps=zeros(ncol,ncol);
for j=jmin:jmax,
    x=exp(2*pi*i*j*(0:ncol-1)'/ncol);
    f=fs*j/ncol;
    gamma=.5*(9e-37./f.^5+4.5e-43./f+3.24e-46*(1+(f/500).^2));
    Heps=Heps+x*x'/gamma;
end;
Heps=Heps/(fs*ncol);
fprintf(' done.\n');

% -- schur
Heps=g*Heps*g';
fprintf('computing schur...');
fflush(1);
[q d]=schur(Heps);
fprintf(' done.\n');

[evg idx]=sort(diag(d));
clear d

fprintf('computing eigen vectors...');
fflush(1);
evect=g*q;
fprintf(' done.\n');

% -- pick up first and second vector

eps1=evect(:,idx(ncol));
eps1=eps1/norm(eps1);
eps2=evect(:,idx(ncol-4));
eps2=eps2/norm(eps2);
eps3=evect(:,idx(ncol-8));
eps3=eps3/norm(eps3);
eps4=evect(:,idx(ncol-12));
eps4=eps4/norm(eps4);

% -- alf

x=zeros(ncol,1);
mid=ncol/2;
x(mid+(-Nalf/2:Nalf/2))=(-Nalf/2:Nalf/2);
%alf1=conv(wf,x);
%alf1=alf1(ncol+(-mid+1:mid));
alf1=glevinson(wfilter,x');
%alf1=fs*alf1';
alf1=alf1/norm(alf1);

x=zeros(ncol,1); 
x(mid+(-Nalf/2:Nalf/2))=ones(Nalf+1,1);
%alf2=conv(wf,x);
%alf2=alf2(ncol+(-mid+1:mid));
alf2=glevinson(wfilter,x');
%alf2=fs*alf2';
alf2=alf2/norm(alf2);

t=((1:ncol)-mid)/fs;

% -- compute spectra

f=fs*(0:nf/2-1)/nf;

if (nf-ncol)>0
  eps1f=fft([eps1;zeros(nf-ncol,1)]);
  eps2f=fft([eps2;zeros(nf-ncol,1)]);
  eps3f=fft([eps3;zeros(nf-ncol,1)]);
  eps4f=fft([eps4;zeros(nf-ncol,1)]);
  alf1f=fft([alf1;zeros(nf-ncol,1)]);
  alf2f=fft([alf2;zeros(nf-ncol,1)]);
else
  eps1f=fft(eps1);
  eps2f=fft(eps2);
  eps3f=fft(eps3);
  eps4f=fft(eps4);
  alf1f=fft(alf1);
  alf2f=fft(alf2);
end;

% -- output figure

if debug==0
  gset term posts eps enhanced color dashed "helvetica" 20
  gset output "figure4.eps";
end;

% -- plotting...
multiplot(2,2);

subplot(2,2,1);
title('(a) excess power mon');
xlabel('time (ms)');
ylabel('amplitude');
eval(sprintf('gset xrange [%f:%f]',-10,10));
gset xtics -10,5,10;
grid("on");
plot(t*1e3,eps1);
hold on
plot(t*1e3,eps2,'--');
hold off

% relax
gset xtics autofreq
gset ytics autofreq
gset autoscale x
gset autoscale y

subplot(2,2,2);
gset xrange [0:2000]
gset xtics 0,1000,2000
title('(b)');
xlabel('frequency (Hz)');
ylabel('ampl. spec.');
plot(f,abs(eps1f(1:nf/2)));
hold on
plot(f,abs(eps2f(1:nf/2)),'--');
plot(f,abs(eps3f(1:nf/2)),'.-');
plot(f,abs(eps4f(1:nf/2)),'+-');
hold off

subplot(2,2,3);
title('(c) local linear fit');
xlabel('time (ms)');
ylabel('amplitude');
eval(sprintf('gset xrange [%f:%f]',-5,5));
eval(sprintf('gset yrange [%f:%f]',-.25,.25));
gset xtics -5,1,5;
gset ytics -1,.1,1
grid("on");
plot(t*1e3,alf1);
hold on
plot(t*1e3,alf2,'--');
hold off

% -- relax
gset xtics autofreq
gset ytics autofreq
gset autoscale x
gset autoscale y

subplot(2,2,4);
gset xrange [0:2000]
gset xtics 0,1000,2000
title('(d)');
xlabel('frequency (Hz)');
ylabel('ampl. spec.');
plot(f,abs(alf1f(1:nf/2)));
hold on
plot(f,abs(alf2f(1:nf/2)),'--');
hold off

% -- relax
oneplot();
gset autoscale x
gset autoscale y
gset term x11