% Copyright (C) 2001 Eric Chassande-Mottin
%
% This program is free software and may be used for any purpose.  This
% copyright notice must be maintained. Eric Chassande-Mottin is not responsible
% for the consequences of using this software.
%
% Based on:
%    levinson.m
%    Copyright (C) 1999 Paul Kienzle
%    yulewalker.m
%    Author:  PAK <pkienzle@kienzle.powernet.co.uk>
%    Copyright (C) 1995 (GPL)
%    Friedrich Leisch <Friedrich.Leisch@ci.tuwien.ac.at>

% usage:  [x, y] = levinson (acf [, b])
%
% Use the Durbin-Levinson algorithm to solve 
%
% a) the standard Yule Walker equations
%         toeplitz(acf(1:n)) * x = -acf(2:n+1)
% verify???
% The solution [1, x'] is the denominator of an all pole filter
% approximation to the signal x which generated the autocorrelation
% function acf.  
% Use freqz(sqrt(v),[1 x']) to plot the power spectrum.
% Returns 
%   a=[1, x'] the denominator filter coefficients. 
%   v= variance of the white noise = square of the numerator constant
%   ref = reflection coefficients = coefficients of the lattice
%         implementation of the filter
%
% acf is the autocorrelation function for lags 0 to n+1.
%
% b) the Toeplitz system with a general right hand side
%         toeplitz(acf(1:n)) * x = b
%
% acf is the autocorrelation function for lags 0 to n.
% b is the right hand side 
%
% Ref: G. Golub and C. Van Loan, "Matrix computations", second ed.,
% John Hopkins Univ. Press, 1993. [chapter 4, p. 185] 

% Author:  ecm@obs-nice.fr


function [x, y] = glevinson (acf, b)
  
  if( columns (acf) > 1 ) error('ACF must be a column vector'); end
  
% standard yule walker equations
  if (nargin == 1) 
    %k=0
    n=length(acf)-1;
    y= -acf(2)/acf(1);
    a = [y];
    beta= (1 - a^2) * acf(1); % beta_{1}
    
    %k=1:n-1
    for k=1:n-1,
      a=-(acf(k+2)+y*acf(k+1:-1:2))/beta;
	y=[y+a*y(k:-1:1),a];
	beta=beta*(1-a^2);
    end;
    x=y;
  end

% toeplitz system with general right hand side
  if (nargin == 2)

%     n = length(b); 
%     if length(acf)<n
%       error('ACF must have length(B) rows');
%     end;
%     
%     %k=0
%     x= b(1)/acf(1);
%     y= -acf(2)/acf(1);
%     a = [y];
%     beta= (1 - a^2) * acf(1); % beta_{1}
%     
%     %k=1:n-1
%     for k=1:n-1,
%       m=(b(k+1)-x*acf(k+1:-1:2))/beta;
%       x=[x+m*y(k:-1:1),m];
%       if (k<n-1),
% 	a=-(acf(k+2)+y*acf(k+1:-1:2))/beta;
% 	y=[y+a*y(k:-1:1),a];
% 	beta=beta*(1-a^2);
%       end;
%     end;
  
    [col,n]=size(b);
    if length(acf)<n
      error('ACF and B must have the same number of columns');
    end;
    
    %k=0
    x= b(:,1)/acf(1);
    y= -acf(2)/acf(1);
    a = [y];
    beta= (1 - a^2) * acf(1); % beta_{1}
    
    %k=1:n-1
    for k=1:n-1,
      m=(b(:,k+1)-x*acf(k+1:-1:2))/beta;
      x=[x+m*y(k:-1:1),m];
      if (k<n-1),
	a=-(acf(k+2)+y*acf(k+1:-1:2))/beta;
	y=[y+a*y(k:-1:1),a];
	beta=beta*(1-a^2);
      end;
    end;
  
  end;

endfunction
 
