
% PREPROCESS THE DFM waveforms
% compute and (over)whiten the DFM catalog
% evaluate SNR@1MPc

clear;

% -- init
debug=1;              % debug messages yes=1, no=0
datafiledir=""

if debug==1
  fprintf('loading supernovae lib...\n');
  fflush(1);
end;

% -- load the data
loadstr=["load " datafiledir "snvdata.mat"]
eval(loadstr);

if debug==1
  fprintf('done.\n');
end;

[ncol nfile]=size(snvdata);

% -- whiten and over-whiten data

nf=2^(nextpow2(ncol));
psd=zeros(nf,1);
acftmp=zeros(nf,1);
f=(0:nf/2)/nf*fs;

% use of official Virgo sensitivity
% .5 factor because the given curve is one sided
psd(1:nf/2+1)=.5*(9e-37./f.^5+4.5e-43./f+3.24e-46*(1+(f/500).^2));
psd(1)=psd(2); % remove singularity @DC
psd(nf/2+2:nf)=flipud(psd(2:nf/2));
acftmp=fs*real(ifft(psd));

hnoisef=sqrt(psd);
wfilter=fs*real(ifft(hnoisef));

acf=acftmp(1:ncol);

if debug==1
  fprintf('whiten supernovae signals...\n');
  fflush(1);
end;

snvdataw=glevinson(wfilter,snvdata'); % whiten
snvdataw=fs*snvdataw';

if debug==1
  fprintf('done.\n');
end;

if debug==1
  fprintf('over-whiten supernovae signals...\n');
  fflush(1);
end;

snvdataf=glevinson(acf,snvdata');  % overwhiten
snvdataf=snvdataf';

if debug==1
  fprintf('done.\n');
end;

% -- compute max size using over-whiten snvwvf

if debug==1
  fprintf('center whiten signals...\n');
  fflush(1);
end;

threshold=1e-4;
tc=zeros(nfile,1);
tm=zeros(nfile,1);
tp=zeros(nfile,1);
tmax=zeros(nfile,1);
tpsd=200;
for icol=1:nfile,
  x=snvdataf(:,icol);
  xmin=min(x);
  tc(icol)=find(x==xmin);
  pwr=sqrt(cumsum(x(tc(icol):-1:tpsd).^2));
  n=length(pwr);
  tm(icol)=min(find(pwr>(1.0-threshold)*pwr(n)));
  pwr=sqrt(cumsum(x(tc(icol):(ncol-tpsd)).^2));
  n=length(pwr);
  tp1=min(find(diff(pwr/pwr(n))<eps));
  tp2=min(find(pwr>(1.0-threshold)*pwr(n)));
  tp(icol)=max([tp1 tp2]);
end;
tmmax=max(tm);
tpmax=max(tp);
dtmax=tmmax+tpmax+1;

if debug==1
  fprintf('done.\n');
end;

% -- compute (amplitude) snr 
snr=zeros(nfile,1);
for icol=1:nfile
  tau=tc(icol)+(-tm(icol):tp(icol));
  snr(icol)=norm(snvdataw(tau,icol))/sqrt(fs);
end;
norm_coef=snr;

% -- center and normalize waveforms
% rule: synchro all waveforms around the min of large negative bump, all snv wvf have.
snvdatac=zeros(dtmax,nfile);  % original waveforms
snvdatawc=zeros(dtmax,nfile); % whitened waveforms
snvdatafc=zeros(dtmax,nfile); % "over-whitened" waveforms
for icol=1:nfile,
  if debug==2,
    printf('%d\n',icol);fflush(1);
  end;
  x=snvdata(:,icol);
  y=snvdataw(:,icol);
  z=snvdataf(:,icol);
  tau=-tm(icol):tp(icol);
  snvdatawc(tmmax+1+tau,icol)=y(tc(icol)+tau)/norm_coef(icol);
  snvdatafc(tmmax+1+tau,icol)=z(tc(icol)+tau)/norm_coef(icol);
  snvdatac(tmmax+1+tau,icol)=x(tc(icol)+tau)/norm_coef(icol);
  n=length(tau);
  idx=1:tmmax+tau(1); ni=length(idx);
  if (ni>0)
    snvdatac(idx,icol)=snvdatac(idx(ni)+1,icol)*ones(ni,1);
  end;
  idx=tmmax+tau(n):dtmax; ni=length(idx);
  if (ni>0)
    snvdatac(idx,icol)=snvdatac(idx(1)-1,icol)*ones(ni,1);
  end;
end;

% -- write output file
if debug==1
  fprintf('writing snvdatacenter.mat...\n');
  fflush(1);
end;

eval(["save -mat-binary " datafiledir "snvdatacenter.mat snvdatac snvdatafc snvdatawc psd snr norm_coef fs threshold"]);

if debug==1
  fprintf('done.\n');
  fflush(1);
end;

if debug==1
 fprintf('measured mean snr= %f\n',mean(snr));
end;

